@@ -80,6 +80,8 @@ class AcousticField(ABC):
 
             self.params = {
                 'c0': params.acoustic['c0'],
+                'voltage': params.acoustic['voltage'],
+                'sensitivity': params.acoustic['sensitivity'],
                 'Foc': params.acoustic['Foc'],
                 'N_piezoFocal': params.acoustic['N_piezoFocal'],
                 'f_US': int(float(params.acoustic['f_US'])),
@@ -228,12 +230,12 @@ class AcousticField(ABC):
             elif self.params['typeSim'] == TypeSim.KWAVE.value:
                 if self.params["dim"] == Dim.D2.value:
                     try:
-                        field = self._generate_2Dacoustic_field_KWAVE(isGpu, show_log)
+                        field = self._generate_acoustic_field_KWAVE_2D(isGpu, show_log)
                     except Exception as e:
                         raise RuntimeError(f"Failed to generate 2D acoustic field: {e}")
                     self.field = calculate_envelope_squared(field, isGpu)
                 elif self.params["dim"] == Dim.D3.value:
-                    field = self._generate_3Dacoustic_field_KWAVE(isGpu, show_log)
+                    field = self._generate_acoustic_field_KWAVE_3D(isGpu, show_log)
                     self.field = calculate_envelope_squared(field, isGpu)
             elif self.params['typeSim'] == TypeSim.HYDRO.value:
                 raise ValueError("Cannot generate field for Hydrophone simulation, load exciting acquisitions.")
@@ -420,18 +422,49 @@ class AcousticField(ABC):
             print(f"Error creating animation: {e}")
             return None
 
-    def show(self):
+    def show(self, use_dB=False, reference=1e6,Vmax=None):
         """
-        Display the string representation of the AcousticField object.
+        Display the maximum intensity projection of the acoustic field envelope.
+
+        Parameters:
+        - use_dB (bool): If True, display in dB relative to the reference pressure.
+        - reference (float): Reference pressure in Pa for dB calculation (default: 1 MPa).
         """
         try:
-            if self.field is None:  
+            if self.field is None:
                 raise ValueError("Field data is not available. Please generate or load the field first.")
             if self.field.min() < 0:
                 raise ValueError("Calculation of the envelope has not been performed. Please generate the envelope first.")
+
+            # Convertir l'enveloppe au carré en amplitude (Pa) en prenant la racine carrée
+            envelope_amplitude = np.sqrt(self.field)
+
+            if use_dB:
+                # Convertir en dB re reference (Pa)
+                envelope_dB = 20 * np.log10(envelope_amplitude / reference)
+                data_to_show = envelope_dB
+                unit_label = f'dB re {reference / 1e6} MPa'
+                if Vmax is not None:
+                    vmax = Vmax
+                else:
+                    vmax = 0
+
+            else:
+                # Convertir en MPa
+                envelope_amplitude_mpa = envelope_amplitude / 1e6
+                data_to_show = envelope_amplitude_mpa
+                unit_label = 'MPa'
+                if Vmax is not None:
+                    vmax = Vmax
+                else:
+                    vmax = 0.85*np.max(envelope_amplitude_mpa)
+
             plt.figure(figsize=(10, 6))
-            plt.imshow(self.field.max(axis=0), extent=(self.params['Xrange'][0]*1000, self.params['Xrange'][-1]*1000, self.params['Zrange'][-1]*1000, self.params['Zrange'][0]*1000), aspect='equal', cmap='jet', vmin=0, vmax=self.field.max()*0.7)
-            plt.colorbar(label='Envelope Amplitude')
+            plt.imshow(data_to_show.max(axis=0),
+                    extent=(self.params['Xrange'][0] * 1000, self.params['Xrange'][1] * 1000,
+                            self.params['Zrange'][1] * 1000, self.params['Zrange'][0] * 1000),
+                    aspect='equal', cmap='jet', vmin=0, vmax=vmax)
+            plt.colorbar(label=f'Envelope Amplitude ({unit_label})')
             plt.title('Maximum Intensity Projection of Acoustic Field Envelope')
             plt.xlabel('X (mm)')
             plt.ylabel('Z (mm)')
@@ -440,6 +473,7 @@ class AcousticField(ABC):
             print(f"Error in show method: {e}")
             raise
 
+
     ## PRIVATE METHODS ##
 
     def _generate_burst_signal(self):
@@ -460,7 +494,7 @@ class AcousticField(ABC):
             print(f"Error in __generate_burst_signal method: {e}")
             raise
 
-    def _generate_2Dacoustic_field_KWAVE(self, isGPU=True if config.get_process() == 'gpu' else False, show_log=True):
+    def _generate_acoustic_field_KWAVE_2D(self, isGPU=True if config.get_process() == 'gpu' else False, show_log=True):
         """
         Base function to generate a 2D acoustic field using k-Wave.
         Handles common setup, simulation, and post-processing.
@@ -485,6 +519,8 @@ class AcousticField(ABC):
             kgrid = kWaveGrid([Nx, Nz], [dx, dx])
             kgrid.setTime(self.kgrid.Nt, 1 / self.params['f_AQ'])
 
+            print(f"Grid size: Nx={Nx}, Nz={Nz}, dx={dx*1000:.3f} mm, dt={kgrid.dt*1e9:.3f} ns, Nt={kgrid.Nt}, f_AQ={self.params['f_AQ']/1e6:.2f} MHz")
+
             source = kSource()
             source.p_mask = np.zeros((Nx, Nz))
 
@@ -533,7 +569,7 @@ class AcousticField(ABC):
                 return reshape_field(data, [factorT, factorX, factorZ])
             else:
                 return data
-
+            
         except Exception as e:
             print(f"Error generating 2D acoustic field: {e}")
             return None
@@ -543,11 +579,29 @@ class AcousticField(ABC):
         Generate a 3D acoustic field using k-Wave.
         """
         try:
+            # --- 1. Grid setup (common) ---
+            dx = self.params['dx']
+            if dx >= self.params['element_width']:
+                dx = self.params['element_width'] / 2
+                Nx = int(round((self.params['Xrange'][1] - self.params['Xrange'][0]) / dx))
+                Nz = int(round((self.params['Zrange'][1] - self.params['Zrange'][0]) / dx))
+            else:
+                Nx = self.params['Nx']
+                Nz = self.params['Nz']
+
+            # --- 2. Time and space factors (common) ---
+            factorT = int(np.ceil(self.params['f_AQ'] / self.params['f_saving']))
+            factorX = int(np.ceil(Nx / self.params['Nx']))
+            factorZ = int(np.ceil(Nz / self.params['Nz']))
+
+            kgrid = kWaveGrid([Nx, Nz], [dx, dx])
+            kgrid.setTime(self.kgrid.Nt, 1 / self.params['f_AQ'])
+
             source = kSource()
             source.p_mask = np.zeros((self.params['Nx'], self.params['Ny'], self.params['Nz']))
 
             # Appel à la méthode spécialisée
-            self._SetUpSource(source, self.params['Nx'], self.params['dx'], 1)  # factorT=1 pour simplifier
+            self._SetUpSource(source, self.params['Nx'], self.params['dx'], factorT)  # factorT=1 pour simplifier
 
             sensor = kSensor()
             sensor.mask = np.ones((self.params['Nx'], self.params['Ny'], self.params['Nz']))
@@ -568,7 +622,7 @@ class AcousticField(ABC):
             )
 
             sensor_data = kspaceFirstOrder3D(
-                kgrid=self.kgrid,
+                kgrid=kgrid,
                 medium=self.medium,
                 source=source,
                 sensor=sensor,
@@ -576,7 +630,11 @@ class AcousticField(ABC):
                 execution_options=execution_options,
             )
 
-            return sensor_data['p'].reshape(self.kgrid.Nt, self.params['Nz'], self.params['Ny'], self.params['Nx'])
+            data = sensor_data['p'].reshape(kgrid.Nt, Nz, Nx)
+            if factorT != 1 or factorX != 1 or factorZ != 1:
+                return reshape_field(data, [factorT, factorX, factorZ])
+            else:
+                return data
 
         except Exception as e:
             print(f"Error generating 3D acoustic field: {e}")
