@@ -123,17 +123,21 @@ class Tomography(Experiment):
         ax2.tick_params(axis='y', colors='r')
         plt.show()
 
-    def plot_angle_frequency_distribution(self, patterns, num_elements=192):
+    def plot_angle_frequency_distribution(self):
         """
         Plots the distribution of angles and spatial frequencies from a list of patterns.
         Args:
             patterns (list): List of strings in the format "hex_angle".
             num_elements (int): Number of elements in each pattern (default is 192).
         """
-        freq_bins = [2, 4, 6, 8, 12, 16, 24, 32, 48, 64, 96, 192]
+        if self.patternList is None:
+            raise ValueError("patternList is not initialized. Please load or generate the active list first.")
+        
+        num_elements = self.params.acoustic['num_elements']
+        divs = [d for d in range(2, num_elements + 1) if num_elements % d == 0 and d % 2 == 0]
         angles = []
         freqs = []
-        for p in patterns:
+        for p in self.patterns:
             hex_part, angle_str = p.split('_')
             # Récupérer l'angle
             sign = -1 if angle_str[0] == '1' else 1
@@ -141,12 +145,9 @@ class Tomography(Experiment):
             angles.append(angle)
             # Récupérer fréquence spatiale
             bits = np.array([int(b) for b in bin(int(hex_part, 16))[2:].zfill(num_elements)])
-            # Cas spécial : pattern "192 on" (tous les bits à 1)
-            if np.all(bits == 1):
-                freqs.append(192)
-                continue
+
             # Chercher la plus petite taille de bloc divisant num_elements
-            for block_size in freq_bins:
+            for block_size in divs:
                 half_block = block_size // 2
                 block = np.array([0]*half_block + [1]*half_block)
                 reps = num_elements // block_size
@@ -167,11 +168,11 @@ class Tomography(Experiment):
         axes[0].set_title("Distribution des angles")
         axes[0].set_xticks(np.arange(-20, 21, 2))
         # Histogramme des fréquences spatiales
-        bins = np.append(freq_bins, 193)
+        bins = np.append(divs, 193)
         axes[1].hist(freqs, bins=bins, color='salmon', edgecolor='black', rwidth=0.8)
         axes[1].set_xscale('log')
-        axes[1].set_xticks(freq_bins)
-        axes[1].set_xticklabels(freq_bins)
+        axes[1].set_xticks(divs)
+        axes[1].set_xticklabels(divs)
         axes[1].set_xlabel("Taille du bloc / fréquence spatiale")
         axes[1].set_ylabel("Nombre de patterns")
         axes[1].set_title("Distribution des fréquences spatiales")
@@ -219,10 +220,9 @@ class Tomography(Experiment):
         """
         if N < 1:
             raise ValueError("N must be a positive integer.")
-        patterns = self._generate_patterns(N)
-        if not self._check_patterns(patterns):
+        self.patterns = self._generate_patterns(N)
+        if not self._check_patterns(self.patterns):
             raise ValueError("Generated patterns failed validation.")
-        return patterns
 
     def _generate_patterns(self, N):
         def format_angle(a):
@@ -230,79 +230,70 @@ class Tomography(Experiment):
 
         def bits_to_hex(bits):
             bit_string = ''.join(str(b) for b in bits)
-            return f"{int(bit_string, 2):0{self.params['num_elements']//4}x}"
+            bit_string = bit_string.zfill(len(bits))
+            hex_string = ''.join([f"{int(bit_string[i:i+4], 2):x}" for i in range(0, len(bit_string), 4)])
+            return hex_string
 
-        num_elements = self.params['num_elements']
-
-        # 1. Générer le pattern "tous les éléments activés" pour tous les angles
-        all_on_bits = np.ones(num_elements, dtype=int)
-        all_on_hex = bits_to_hex(all_on_bits)
-        all_on_pairs = [f"{all_on_hex}_{format_angle(angle)}" for angle in range(-20, 21)]
-        results = set(all_on_pairs)
-
-        # 2. Calculer combien de patterns équilibrés il reste à générer
-        remaining_N = N - len(all_on_pairs)
-        if remaining_N <= 0:
-            return list(results)[:N]
-
-        # 3. Trouver tous les diviseurs de num_elements (sauf 1 et num_elements)
-        divs = [d for d in range(2, num_elements) if num_elements % d == 0]
+        num_elements = self.params.acoustic['num_elements']
         angle_choices = list(range(-20, 21))
-        nb_freq = len(divs)
-        max_per_block = int(np.ceil(remaining_N / nb_freq)) if nb_freq > 0 else 0
 
-        for block_size in divs:
-            half_block = block_size // 2
-            block = np.array([0]*half_block + [1]*half_block)
-            reps = num_elements // block_size
-            base_pattern = np.tile(block, reps)
-            n_shifts = 1 if block_size == num_elements else block_size
-            all_shifted = [np.roll(base_pattern, shift) for shift in range(n_shifts)]
-            np.random.shuffle(all_shifted)
-            count = 0
-            for pattern_bits in all_shifted:
-                available_angles = angle_choices.copy()
-                np.random.shuffle(available_angles)
-                for angle in available_angles:
-                    hex_pattern = bits_to_hex(pattern_bits)
-                    pair = f"{hex_pattern}_{format_angle(angle)}"
-                    if pair not in results:
-                        results.add(pair)
-                        count += 1
-                        if count >= max_per_block:
-                            break
-                if count >= max_per_block:
-                    break
+        # 1. Trouver TOUS les diviseurs PAIRS de num_elements (y compris num_elements)
+        divs = [d for d in range(2, num_elements + 1) if num_elements % d == 0 and d % 2 == 0]
+        if not divs:
+            print(f"Aucun diviseur pair trouvé pour num_elements = {num_elements}")
+            return []
+
+        results = []
+
+        # 2. Générer N patterns en tirant aléatoirement parmi les divs et les angles
+        for _ in range(N):
+            # Tirer un diviseur aléatoire (y compris num_elements)
+            block_size = np.random.choice(divs)
+
+            if block_size == num_elements:
+                # Cas spécial : pattern "tous activés"
+                pattern_bits = np.ones(num_elements, dtype=int)
+            else:
+                # Cas général : pattern équilibré
+                half_block = block_size // 2
+                block = np.array([0] * half_block + [1] * half_block)
+                reps = num_elements // block_size
+                base_pattern = np.tile(block, reps)
+                # Tirer un décalage aléatoire
+                shift = np.random.randint(0, block_size)
+                pattern_bits = np.roll(base_pattern, shift)
+
+            # Convertir en hex et choisir un angle aléatoire
+            hex_pattern = bits_to_hex(pattern_bits)
+            angle = np.random.choice(angle_choices)
+            pair = f"{hex_pattern}_{format_angle(angle)}"
+
+            results.append(pair)
+
+        return results
 
-        # 4. Compléter si nécessaire
-        results = list(results)
-        if len(results) < N:
-            extra_needed = N - len(results)
-            results += list(np.random.choice(results, extra_needed, replace=True))
 
-        np.random.shuffle(results)
-        return results[:N]
 
 
     def _check_patterns(self, patterns):
         for p in patterns:
             hex_part = p.split('_')[0]
-            bits = np.array([int(b) for b in bin(int(hex_part, 16))[2:].zfill(self.params['num_elements'])])
-            if len(bits) != self.params['num_elements']:
+            bits = np.array([int(b) for b in bin(int(hex_part, 16))[2:].zfill(self.params.acoustic['num_elements'])])
+            if len(bits) != self.params.acoustic['num_elements']:
                 print(f"Erreur longueur: {p}")
                 return False
             if np.all(bits == 1):
                 continue
-            if np.sum(bits) != self.params['num_elements'] // 2:
+            if np.sum(bits) != self.params.acoustic['num_elements'] // 2:
                 print(f"Erreur équilibre 0/1: {p}")
                 return False
             valid = False
-            for block_size in range(2, self.params['num_elements']+1, 2):
-                if self.params['num_elements'] % block_size != 0:
+            for block_size in range(2, self.params.acoustic['num_elements']+1, 2):
+                if self.params.acoustic['num_elements'] % block_size != 0:
                     continue
                 half_block = block_size // 2
                 block = np.array([0]*half_block + [1]*half_block)
-                reps = self.params['num_elements'] // block_size
+                reps = self.params.acoustic['num_elements'] // block_size
                 expected_pattern = np.tile(block, reps)
                 if any(np.array_equal(np.roll(expected_pattern, shift), bits) for shift in range(block_size)):
                     valid = True
